"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pathNormalize = pathNormalize;
exports.pathDirectory = pathDirectory;
exports.pathBasename = pathBasename;
exports.pathExtension = pathExtension;
exports.pathJoin = pathJoin;
function __assignType(fn, args) {
    fn.__type = args;
    return fn;
}
/**
 * Normalizes the given path.
 * Removes duplicate slashes, removes trailing slashes, adds a leading slash.
 */
function pathNormalize(path) {
    path = path[0] !== '/' ? '/' + path : path;
    path = path.length > 1 && path[path.length - 1] === '/' ? path.slice(0, -1) : path;
    return path.replace(/\/+/g, '/');
}
pathNormalize.__type = ['path', 'pathNormalize', 'P&2!&/"'];
/**
 * Returns the directory (dirname) of the given path.
 */
function pathDirectory(path) {
    if (path === '/')
        return '/';
    const lastSlash = path.lastIndexOf('/');
    return lastSlash <= 0 ? '/' : path.slice(0, lastSlash);
}
pathDirectory.__type = ['path', 'pathDirectory', 'P&2!&/"'];
/**
 * Returns the basename of the given path.
 */
function pathBasename(path) {
    const lastSlash = path.lastIndexOf('/');
    return lastSlash === -1 ? path : path.slice(lastSlash + 1);
}
pathBasename.__type = ['path', 'pathBasename', 'P&2!&/"'];
/**
 * Returns the extension of the given path.
 */
function pathExtension(path) {
    const basename = pathBasename(path);
    const lastDot = basename.lastIndexOf('.');
    return lastDot === -1 ? '' : basename.slice(lastDot + 1);
}
pathExtension.__type = ['path', 'pathExtension', 'P&2!&/"'];
function pathJoin(...paths) {
    return '/' + paths
        .map(__assignType(v => pathNormalize(v).slice(1), ['v', '', 'P"2!"/"']))
        .filter(__assignType(v => !!v, ['v', '', 'P"2!"/"']))
        .join('/');
}
pathJoin.__type = ['paths', 'pathJoin', 'P&@2!&/"'];
//# sourceMappingURL=path.js.map