/*
 * Deepkit Framework
 * Copyright (C) 2021 Deepkit UG, Marc J. Schmidt
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the MIT License.
 *
 * You should have received a copy of the MIT License along with this program.
 */
import { arrayRemoveItem } from './array.js';
const __ΩAsyncSubscriber = ['T', 'event', '', 'AsyncSubscriber', 'b!Pe"!2"P$`$J/#w$y'];
const __ΩSubscriber = ['T', 'event', '', 'Subscriber', 'b!Pe"!2"P$`$J/#w$y'];
const __ΩAsyncEventSubscription = ['', 'unsubscribe', 'AsyncEventSubscription', 'PP$/!4"Mw#y'];
export { __ΩAsyncEventSubscription as __ΩAsyncEventSubscription };
const __ΩEventSubscription = ['', 'unsubscribe', 'EventSubscription', 'PP$/!4"Mw#y'];
export { __ΩEventSubscription as __ΩEventSubscription };
let asyncId = 0;
export class EmitterEvent {
    constructor() {
        this.id = asyncId++;
        this.stopped = false;
        this.propagationStopped = false;
    }
    /**
     * Stop propagating the event to subsequent event listeners.
     */
    stopPropagation() {
        this.propagationStopped = true;
    }
    /**
     * Signal the emitter that you want to abort.
     * Subsequent event listeners will still be called.
     */
    stop() {
        this.stopped = true;
    }
}
EmitterEvent.__type = ['id', function () { return asyncId++; }, 'stopped', function () { return false; }, 'propagationStopped', function () { return false; }, 'stopPropagation', 'stop', 'EmitterEvent', '!3!9>")3#>$)3%>&P"0\'P"0(5w)'];
export class EventEmitter {
    constructor(parent) {
        this.parent = parent;
        this.subscribers = [];
    }
    subscribe(callback) {
        this.subscribers.push(callback);
        return {
            unsubscribe: () => {
                arrayRemoveItem(this.subscribers, callback);
            }
        };
    }
    emit(event) {
        if (this.parent)
            this.parent.emit(event);
        if (event.propagationStopped)
            return;
        for (const subscriber of this.subscribers) {
            subscriber(event);
            if (event.propagationStopped)
                return;
        }
    }
    hasSubscriptions() {
        return this.subscribers.length > 0;
    }
}
EventEmitter.__type = ['T', () => __ΩSubscriber, 'subscribers', function () { return []; }, () => EventEmitter, 'parent', 'constructor', () => __ΩSubscriber, 'callback', () => __ΩEventSubscription, 'subscribe', 'event', 'emit', 'hasSubscriptions', 'EventEmitter', 'b!e!!o""F3#<>$PP"7%2&8<"0\'Pe"!o("2)n*0+Pe"!2,$0-P)0.5w/'];
export class AsyncEventEmitter {
    constructor(parent) {
        this.parent = parent;
        this.subscribers = [];
    }
    subscribe(callback) {
        this.subscribers.push(callback);
        return {
            unsubscribe: () => {
                arrayRemoveItem(this.subscribers, callback);
            }
        };
    }
    async emit(event) {
        if (this.parent)
            await this.parent.emit(event);
        if (event.propagationStopped)
            return;
        for (const subscriber of this.subscribers) {
            await subscriber(event);
            if (event.propagationStopped)
                return;
        }
    }
    hasSubscriptions() {
        return this.subscribers.length > 0;
    }
}
AsyncEventEmitter.__type = ['T', () => __ΩAsyncSubscriber, 'subscribers', function () { return []; }, () => AsyncEventEmitter, 'parent', 'constructor', () => __ΩAsyncSubscriber, 'callback', () => __ΩAsyncEventSubscription, 'subscribe', 'event', 'emit', 'hasSubscriptions', 'AsyncEventEmitter', 'b!e!!o""F3#<>$PP"7%2&8<"0\'Pe"!o("2)n*0+Pe"!2,$`0-P)0.5w/'];
//# sourceMappingURL=emitter.js.map