function __assignType(fn, args) {
    fn.__type = args;
    return fn;
}
export const nextTick = typeof requestAnimationFrame !== 'undefined'
    ? __assignType((cb) => requestAnimationFrame(cb), ['', 'cb', 'PP$/!2""/!']) : __assignType((cb) => setTimeout(cb), ['', 'cb', 'PP$/!2""/!']);
export const clearTick = typeof requestAnimationFrame !== 'undefined'
    ? __assignType((id) => cancelAnimationFrame(id), ['id', '', 'P"2!"/"']) : __assignType((id) => clearTick(id), ['id', '', 'P"2!"/"']);
/**
 * Wraps a function and calls it only `cps` times per frame.
 *
 * This is handy to throttle all kind of rapid calls, like mouse move events or other kind of events.
 *
 * @example
 * ```typescript
 * function expensiveFunction() {
 *     //...
 * }
 *
 * const throttled = throttleTime(expensiveFunction, 5); //5 calls per second max
 *
 * throttled();
 * throttled();
 * throttled();
 * //throttled will here only be called once
 * ```
 */
export function throttleTime(call, cps = 5) {
    let last = Date.now();
    let dirty = false;
    let lastArgs = [];
    let execution = false;
    function tick() {
        const now = Date.now();
        if (!execution && now - last > 1000 / cps) {
            execution = true;
            call(...lastArgs);
            dirty = false;
            last = Date.now();
            execution = false;
        }
        if (dirty) {
            nextTick(tick);
        }
    }
    tick.__type = ['tick', 'P"/!'];
    return __assignType((...args) => {
        dirty = true;
        lastArgs = args;
        tick();
    }, ['args', '', 'P"@2!"/"']);
}
throttleTime.__type = [() => Function, 'call', 'cps', 'args', '', 'throttleTime', 'PPu!2""2#P"@2$$/%/&'];
/**
 * This functions returns a stack that is filled as long as the gate is not activated.
 * Once activated all recorded calls go to given callback and subsequent calls go directly to given callback.
 */
export function bufferedGate(callback) {
    const q = [];
    let activated = false;
    const throttled = throttleTime(async () => {
        if (q.length === 0)
            return;
        for (const t of q) {
            const result = callback(t);
            if (result instanceof Promise) {
                await result;
            }
        }
        //empty the queue
        q.splice(0, q.length);
    });
    return {
        activate: () => {
            activated = true;
            throttled();
        },
        call: __assignType((i) => {
            q.push(i);
            if (activated) {
                throttled();
            }
        }, [() => callback, 'arg', '', 'i', 'PPdi!Ph"!2""/#qe!!!j2$"/#'])
    };
}
bufferedGate.__type = ['arg', '', 'callback', 'bufferedGate', 'PP"2!"/"2#"/$'];
//# sourceMappingURL=reactive.js.map